#!/usr/bin/env python
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.patches import RegularPolygon

def convert_to_ID(my_array, shift, base):
	my_array = my_array + shift
	
	return np.sum(np.multiply(my_array, base **np.arange(my_array.shape[1]-1,-1,-1)), axis=1)


def convert_to_cube_coord(ID_state, shift, base, length):
	cubic_coord = np.ndarray(shape = (ID_state.shape[0], 3))

	for i in range(length-1, -1, -1):
		add = ID_state%base
		cubic_coord[:,i] = add - shift
		ID_state = ID_state // base

	return cubic_coord


def compute_p_out_pheromone(pheromones, nests_coord, shift, base, ID_cases, p_sortie_k, p_sortie_a, p_sortie_b):

	nest_id = convert_to_ID(nests_coord, shift, base)
	pheromone_at_nest = pheromones[np.where(ID_cases == nest_id[:,None])[1]]
	
	p_out = p_sortie_k*(p_sortie_a+pheromone_at_nest**2)/(p_sortie_b+pheromone_at_nest**2)
	
	return p_out


def compute_p_out_recruiters(n_nests, is_recruiting, nest_memories, p_sortie_k, p_sortie_a, p_sortie_b):
	
	recruiters_at_nest = np.zeros(shape = (n_nests, 1 ))
	
	for j in range(0, n_nests):
		recruiters_at_nest[j,0] += np.sum(nest_memories[is_recruiting==1,0]==j)
	
	p_out = p_sortie_k*(p_sortie_a+recruiters_at_nest**2)/(p_sortie_b+recruiters_at_nest**2)

	return p_out

def check_movement(at_food, at_nest, is_recruiting, p_out_food, p_recruiter_leave_nest, p_exit_nest, food_mem, nest_mem):
	
	random_vector_move = np.random.rand(at_food.shape[0],)
	p_move_ant = np.ones(shape = (at_food.shape[0],))
	
	p_move_ant[at_food==1] = p_out_food
	
	p_move_ant[at_nest==1] = p_exit_nest[np.reshape(nest_mem[at_nest==1],(nest_mem[at_nest==1,:].shape[0],)).astype('int'),0]
	
	p_move_ant[is_recruiting == 1] = p_recruiter_leave_nest
	
	move_or_not = random_vector_move < p_move_ant
	
	is_recruiting[(is_recruiting==1) & (move_or_not==1)] = 0
	
	return move_or_not, is_recruiting

def check_homing(ckecked_homing, pheromones_deposits, food_memories, nest_memories, target, positions, food_coords, n_foods, nests_coord, n_nests, food_quality, food_quantity, food_timing, food_duration, food_carriers, first_left, is_recruiting, time_out, total_foods, time_step, record_at_foods, record_at_nests):
	
	at_food_or_nest = np.zeros(shape = (positions.shape[0],))
	at_food = np.zeros(shape = (positions.shape[0],))
	at_nest = np.zeros(shape = (positions.shape[0],))
	
	for j in range(n_foods):
		
		if food_quantity[j] > 0:
		
			ckecked_homing[(positions[:,0] == food_coords[j,0]) & (positions[:,1] == food_coords[j,1]) & (positions[:,2] == food_coords[j,2])] = 1
		
			pheromones_deposits[(positions[:,0] == food_coords[j,0]) & (positions[:,1] == food_coords[j,1]) & (positions[:,2] == food_coords[j,2])] = food_quality[j]
		
			food_memories[(positions[:,0] == food_coords[j,0]) & (positions[:,1] == food_coords[j,1]) & (positions[:,2] == food_coords[j,2])] = j
		
			at_food_or_nest[(positions[:,0] == food_coords[j,0]) & (positions[:,1] == food_coords[j,1]) & (positions[:,2] == food_coords[j,2])] = 1
			at_food[(positions[:,0] == food_coords[j,0]) & (positions[:,1] == food_coords[j,1]) & (positions[:,2] == food_coords[j,2])] = 1
			food_carriers[(positions[:,0] == food_coords[j,0]) & (positions[:,1] == food_coords[j,1]) & (positions[:,2] == food_coords[j,2])] = 1
		
			target[(positions[:,0] == food_coords[j,0]) & (positions[:,1] == food_coords[j,1]) & (positions[:,2] == food_coords[j,2]),:] = nests_coord[nest_memories[(positions[:,0] == food_coords[j,0]) & (positions[:,1] == food_coords[j,1]) & (positions[:,2] == food_coords[j,2]), 0].astype('int'), :]

			record_at_foods[time_step, j] += np.sum((positions[:,0] == food_coords[j,0]) & (positions[:,1] == food_coords[j,1]) & (positions[:,2] == food_coords[j,2]))
			
		else:
			ckecked_homing[(positions[:,0] == food_coords[j,0]) & (positions[:,1] == food_coords[j,1]) & (positions[:,2] == food_coords[j,2])] = 0

			pheromones_deposits[(positions[:,0] == food_coords[j,0]) & (positions[:,1] == food_coords[j,1]) & (positions[:,2] == food_coords[j,2])] = 0

			food_memories[(positions[:,0] == food_coords[j,0]) & (positions[:,1] == food_coords[j,1]) & (positions[:,2] == food_coords[j,2])] = None
			
			at_food_or_nest[(positions[:,0] == food_coords[j,0]) & (positions[:,1] == food_coords[j,1]) & (positions[:,2] == food_coords[j,2])] = 0

			at_food[(positions[:,0] == food_coords[j,0]) & (positions[:,1] == food_coords[j,1]) & (positions[:,2] == food_coords[j,2])] = 0
			food_carriers[(positions[:,0] == food_coords[j,0]) & (positions[:,1] == food_coords[j,1]) & (positions[:,2] == food_coords[j,2])] = 0

			#target[(positions[:,0] == food_coords[j,0]) & (positions[:,1] == food_coords[j,1]) & (positions[:,2] == food_coords[j,2]),:] = np.empty(shape=( (positions[:,0] == food_coords[j,0]) & (positions[:,1] == food_coords[j,1]) & (positions[:,2] == food_coords[j,2]).shape[0] , 3))
	
	for j in range(n_nests):
		
		target[(positions[:,0] == nests_coord[j,0]) & (positions[:,1] == nests_coord[j,1]) & (positions[:,2] == nests_coord[j,2]) & (ckecked_homing[:,0]==1) & (food_carriers==1), :] = food_coords[food_memories[(positions[:,0] == nests_coord[j,0]) & (positions[:,1] == nests_coord[j,1]) & (positions[:,2] == nests_coord[j,2]) & (ckecked_homing[:,0]==1) & (food_carriers==1), 0].astype('int'), :]

		at_food_or_nest[(positions[:,0] == nests_coord[j,0]) & (positions[:,1] == nests_coord[j,1]) & (positions[:,2] == nests_coord[j,2]) & ((food_carriers==1) | (time_out<0))] = 1
		at_nest[(positions[:,0] == nests_coord[j,0]) & (positions[:,1] == nests_coord[j,1]) & (positions[:,2] == nests_coord[j,2]) & ((food_carriers==1) | (time_out<0))] = 1
		
		nest_memories [(positions[:,0] == nests_coord[j,0]) & (positions[:,1] == nests_coord[j,1]) & (positions[:,2] == nests_coord[j,2]) & ((food_carriers==1) | (time_out<0))] = j
		
		count_food = np.unique(food_memories[(positions[:,0] == nests_coord[j,0]) & (positions[:,1] == nests_coord[j,1]) & (positions[:,2] == nests_coord[j,2]) & (food_carriers==1) ].astype('int'), return_counts = True)
		
		total_foods[count_food[0]] += count_food[1]
		
		is_recruiting[(positions[:,0] == nests_coord[j,0]) & (positions[:,1] == nests_coord[j,1]) & (positions[:,2] == nests_coord[j,2]) & (food_carriers==1)] = 1
		
		first_left[(positions[:,0] == nests_coord[j,0]) & (positions[:,1] == nests_coord[j,1]) & (positions[:,2] == nests_coord[j,2]) & (first_left==1) & (food_carriers==1)] = 0
		
		food_carriers[(positions[:,0] == nests_coord[j,0]) & (positions[:,1] == nests_coord[j,1]) & (positions[:,2] == nests_coord[j,2]) & (food_carriers==1)] = 0
		
		record_at_nests[time_step, j] += np.sum((positions[:,0] == nests_coord[j,0]) & (positions[:,1] == nests_coord[j,1]) & (positions[:,2] == nests_coord[j,2]) & ((food_carriers==1) | (time_out<0)))

	return ckecked_homing, pheromones_deposits, food_memories, nest_memories, target, at_food_or_nest, at_food, at_nest, food_carriers, first_left, is_recruiting, time_out, total_foods, record_at_foods, record_at_nests


def homing(positions, targets, bias_1_good_cell_homing, bias_2_good_cells_homing):
	
	positions = positions - targets
	
	check_zeros = positions==0
	check_neg = positions<0
	sum_zeros = np.sum(check_zeros, 1)
	
	biais_homing = np.zeros(shape=(positions.shape[0],6))
	
	biais_homing[(check_zeros[:,0]==1) & (check_neg[:,1]==1), :] = bias_1_good_cell_homing[np.array([0, 1, 2, 3, 4, 5])]
	biais_homing[(check_zeros[:,1]==1) & (check_neg[:,0]==1), :] = bias_1_good_cell_homing[np.array([5, 0, 1, 2, 3, 4])]
	biais_homing[(check_zeros[:,2]==1) & (check_neg[:,0]==1), :] = bias_1_good_cell_homing[np.array([4, 5, 0, 1, 2, 3])]
	biais_homing[(check_zeros[:,0]==1) & (check_neg[:,2]==1), :] = bias_1_good_cell_homing[np.array([3, 4, 5, 0, 1, 2])]
	biais_homing[(check_zeros[:,1]==1) & (check_neg[:,2]==1), :] = bias_1_good_cell_homing[np.array([2, 3, 4, 5, 0, 1])]
	biais_homing[(check_zeros[:,2]==1) & (check_neg[:,1]==1), :] = bias_1_good_cell_homing[np.array([1, 2, 3, 4, 5, 0])]

	biais_homing[(sum_zeros==0) & (check_neg[:,0]==1) & (check_neg[:,1]==1) & (check_neg[:,2]==0), :] = bias_2_good_cells_homing[np.array([0, 1, 2, 3, 4, 5])]
	biais_homing[(sum_zeros==0) & (check_neg[:,0]==1) & (check_neg[:,1]==0) & (check_neg[:,2]==0), :] = bias_2_good_cells_homing[np.array([5, 0, 1, 2, 3, 4])]
	biais_homing[(sum_zeros==0) & (check_neg[:,0]==1) & (check_neg[:,1]==0) & (check_neg[:,2]==1), :] = bias_2_good_cells_homing[np.array([4, 5, 0, 1, 2, 3])]
	biais_homing[(sum_zeros==0) & (check_neg[:,0]==0) & (check_neg[:,1]==0) & (check_neg[:,2]==1), :] = bias_2_good_cells_homing[np.array([3, 4, 5, 0, 1, 2])]
	biais_homing[(sum_zeros==0) & (check_neg[:,0]==0) & (check_neg[:,1]==1) & (check_neg[:,2]==1), :] = bias_2_good_cells_homing[np.array([2, 3, 4, 5, 0, 1])]
	biais_homing[(sum_zeros==0) & (check_neg[:,0]==0) & (check_neg[:,1]==1) & (check_neg[:,2]==0), :] = bias_2_good_cells_homing[np.array([1, 2, 3, 4, 5, 0])]
	
	return biais_homing

def check_limits(positions, potential_moves, ID, limits, shift, base):
	
	limit_moves = np.ndarray(shape = (positions.shape[0], 6))
	
	for j in range(6):
		potential = convert_to_ID(positions + potential_moves[j][None,:], shift, base)
		limit_moves[:,j] = limits[np.where(ID == potential[:,None])[1]]

	return limit_moves

def check_pheromones(positions, potential_moves, ID, pheromones, shift, base):
	
	pheromones_moves = np.ndarray(shape = (positions.shape[0], 6))
	
	for j in range(6):
		potential = convert_to_ID(positions + potential_moves[j][None,:], shift, base)
		pheromones_moves[:,j] = np.reshape(pheromones[np.where(ID == potential[:,None])[1]], (positions.shape[0],))

	return pheromones_moves

def roll_matrix(matrix, roll):
	rows, column_indices = np.ogrid[:matrix.shape[0], :matrix.shape[1]]
	roll[roll < 0] += matrix.shape[1]
	column_indices = column_indices - roll[:,np.newaxis]
	
	return matrix[rows, column_indices]

def select_move(positions, directions, biais, bias_1_good_cell_homing, bias_2_good_cells_homing, foraging_area_connexion, pheromones, homing_sta, homing_target, potential_moves, ponderation_phero, limits, ID, ID2, shift, base):
	
	#d_nat = foraging_area_connexion[ ID2[positions[:,0].astype('int'), positions[:,1].astype('int'), positions[:,2].astype('int')],:]
	
	d_mem = homing(positions, homing_target, bias_1_good_cell_homing, bias_2_good_cells_homing)
	
	d_dir = roll_matrix(biais, directions)

	d_mem = np.multiply(d_mem, homing_sta)
	
	limit_factor = check_limits(positions, potential_moves, ID, limits, shift, base)

	#sum_d = d_nat + d_mem + d_dir
	sum_d = d_mem + d_dir
	
	sum_d = sum_d * limit_factor

	ponderation_pheromone = roll_matrix(ponderation_phero, directions)

	pheromones_potential = np.multiply(check_pheromones(positions, potential_moves, ID, pheromones, shift, base), 1 - homing_sta)
	#pheromones_potential = check_pheromones(positions, potential_moves, ID, pheromones, shift, base)
	
	pheromones_potential = np.multiply(pheromones_potential, ponderation_pheromone)
	
	sum_d += pheromones_potential
	
	sum_d = sum_d**2
	
	p_move = np.cumsum(sum_d, axis = 1) / (1. * np.sum(sum_d, axis=1))[:,None]
	
	rand_move = np.tile(np.random.rand(positions.shape[0],1), (1, 6))

	selected_move = np.argmax(np.greater(p_move, rand_move), axis = 1)

	return selected_move


def update_path(area, positions, ID, deposit, quantity):
	
	area[np.where(ID == positions[:,None])[1]] += deposit * quantity

	return area

def initiate_food(food_distribution, food_position, food_quantity, food_quality, max_dim):
	if food_distribution == "fixed":
		return food_position, food_quantity, food_quality
	elif food_distribution == "random":
		
		food_position = []
		for j in range(0, n_foods):
			food_position.append([])
	
	return food_position, food_quantity, food_quality

def update_food(food_distribution, food_position, food_quantity, food_quality):
	if food_distribution == "fixed":
		return food_position, food_quantity, food_quality



def plot_map(foraging_area, path, pheromone, nests_coordinates, food_coordinates, food_quality, n_ants_in_nest, sim_ID, output_dir):
	
	
	fig = plt.figure(figsize=(15, 15))
	fig.subplots_adjust(left = 0.0, bottom = 0.0, right = 1., top = 1.)

	ax = fig.add_subplot(111)
	ax.set_aspect('equal')
	
	### PLOT MAPS
	
	x_cartesian = foraging_area[:,0]
	y_cartesian = (2. / 3.) * np.sin(np.pi/3.) * (foraging_area[:,1] - foraging_area[:,2])
	
	for x, y in zip(x_cartesian, y_cartesian):
		
		hex = RegularPolygon((x, y), numVertices=6, radius=2. / 3., orientation=np.radians(30), facecolor='white', edgecolor='k')
		ax.add_patch(hex)
	
	
	### PLOT PHEROMONES
	#print np.max(pheromone)
#	path = np.reshape( path / (1*np.max(path)) , (path.shape[0],))
#	max_phero = np.max(pheromone)
#	pheromone = np.reshape( pheromone / (1*np.max(pheromone)) , (pheromone.shape[0],))
#
#	x_cartesian = foraging_area[:,0]
#	y_cartesian = (2. / 3.) * np.sin(np.pi/3.) * (foraging_area[:,1] - foraging_area[:,2])
#
#	for x, y, a in zip(x_cartesian, y_cartesian, pheromone):
#		hex = RegularPolygon((x, y), numVertices=6, radius=2. / 3., orientation=np.radians(30), facecolor='g', alpha=a, edgecolor='k')
#		ax.add_patch(hex)
#		plt.text(x, y, str(np.round(a*max_phero).astype('int')), va = 'center', ha = 'center', zorder=1)

	
	### PLOT NESTS
	
	x_cartesian_nests = nests_coordinates[:,0]
	y_cartesian_nests = (2. / 3.) * np.sin(np.pi/3.) * (nests_coordinates[:,1] - nests_coordinates[:,2])
	
	for x, y, a in zip(x_cartesian_nests, y_cartesian_nests, n_ants_in_nest):
		hex = RegularPolygon((x, y), numVertices=6, radius=2. / 3., orientation=np.radians(30), facecolor='r', edgecolor='k')
		ax.add_patch(hex)
		#plt.text(x, y, str(a), va = 'center', ha = 'center', zorder=2)

	### PLOT FOOD

	x_cartesian_nests = food_coordinates[:,0]
	y_cartesian_nests = (2. / 3.) * np.sin(np.pi/3.) * (food_coordinates[:,1] - food_coordinates[:,2])

	for x, y, q in zip(x_cartesian_nests, y_cartesian_nests, food_quality):
		hex = RegularPolygon((x, y), numVertices=6, radius=2. / 3., orientation=np.radians(30), facecolor='orange', edgecolor='k')
		ax.add_patch(hex)
		#plt.text(x, y, str(q), va = 'center', ha = 'center', zorder=2)
	
	plt.xticks([])
	plt.yticks([])

	plt.xlim([min(x_cartesian)-2, max(x_cartesian)+2])
	plt.ylim([min(y_cartesian)-2, max(y_cartesian)+2])
	plt.savefig(output_dir + '/simulation_' + str(sim_ID) + '.pdf')

	plt.close()


def plot_dynamics(ants_at_foods, ants_at_nests, sim_ID, output_dir):

	fig = plt.figure(figsize=(10, 8))
	fig.subplots_adjust(left = 0.1, bottom = 0.1, right = .95, top = .95)
	
	ax = fig.add_subplot(311)
	plt.plot(np.arange(0,72)*100, np.sum(ants_at_foods, 1)[np.arange(0,72)*100], '.-', color = 'orange', label='At food')
	
	plt.plot(np.arange(0,72)*100, np.sum(ants_at_nests, 1)[np.arange(0,72)*100], '.-', color = 'red', label='At nest')
	
	plt.ylabel('Number of ants')
	plt.xlabel('Time')
	
	plt.legend()
	
	ax = fig.add_subplot(312)
	plt.plot(np.arange(0,72)*100, ants_at_foods[np.arange(0,72)*100,:], '.-')
	
	plt.ylabel('Number of ants')
	plt.xlabel('Time')
	
	ax = fig.add_subplot(313)
	plt.plot(np.arange(0,72)*100, ants_at_nests[np.arange(0,72)*100,:], '.-')

	plt.ylabel('Number of ants')
	plt.xlabel('Time')
	#plt.xlim([min(x_cartesian)-2, max(x_cartesian)+2])
	#plt.ylim([min(y_cartesian)-2, max(y_cartesian)+2])
	plt.savefig(output_dir + '/simulation_' + str(sim_ID) + '_dynamics.pdf')
	
	plt.close()

