#!/usr/bin/env python
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.patches import RegularPolygon
from function_model_v2 import *
import json
import time
import argparse
import os
import datetime

def generate_food(dimension_grid, n_foods, food_quality, food_quantity, food_timing, food_duration, nests_coordinates ):

	space_grid = np.ndarray(shape = ((dimension_grid*2-1)**2, 3))

	space_grid[:,0] = np.repeat(np.arange(-(dimension_grid-1), dimension_grid), dimension_grid*2-1)
	space_grid[:,1] = np.array(range(-(dimension_grid-1), dimension_grid) * (dimension_grid*2-1))
	space_grid[:,2] = -(space_grid[:,0] + space_grid[:,1])

	good_coord = (space_grid[:,2]<dimension_grid) & (space_grid[:,2]>-dimension_grid)

	foraging_area = space_grid[good_coord,:].astype('int')
	
	random_num = list(range(1, foraging_area.shape[0]))
	np.random.shuffle(random_num)

	position_food = []
	quality_food = []
	quantity_food = []
	timing_food = []
	duration_food = []
	k = 0
	
	while len(position_food) < n_foods:
		if (np.prod(np.array([foraging_area[random_num[k],0], foraging_area[random_num[k],1], foraging_area[random_num[k],2]]) == np.array(nests_coordinates)) == 0) & (np.max( np.abs(np.array([foraging_area[random_num[k],0], foraging_area[random_num[k],1], foraging_area[random_num[k],2]]) ) ) < dimension_grid-1):
			position_food.append([foraging_area[random_num[k],0], foraging_area[random_num[k],1], foraging_area[random_num[k],2]])
			quality_food.append(food_quality)
			quantity_food.append(food_quantity)
			timing_food.append(food_timing)
			duration_food.append(food_duration)
			k += 1
		else:
			k += 1
	
	return position_food, quality_food, quantity_food, timing_food, duration_food

if __name__ == '__main__':
	
	ap = argparse.ArgumentParser()
	ap.add_argument("-n", "--nfood", type = int)
	ap.add_argument("-q", "--quanlityfood", type = int)
	ap.add_argument("-a", "--quantityfood", type = int)
	ap.add_argument("-t", "--timingfood", type = int)
	ap.add_argument("-d", "--durationfood", type = int)
	#ap.add_argument("-p", "--nestpositions", type = int)
	ap.add_argument("-g", "--griddimension", type = int)
	args = vars(ap.parse_args())
	
	
	dimension_grid = args["griddimension"]
	n_foods = args["nfood"]
	food_quality = args["quanlityfood"]
	food_quantity = args["quantityfood"]
	food_timing = args["timingfood"]
	food_duration = args["durationfood"]
	nests_coordinates = [ [0, 5, -5], [5, 0, -5], [5, -5, 0], [0, -5, 5], [-5, 0, 5], [-5, 5, 0] ,[0, 0, 0], [-5, 3, 2], [-5, 2, 3], [-4, 2, 2], [4, -2, -2], [5, -2, -3], [5, -3, -2]]
	
	position_food, quality_food, quantity_food, timing_food, duration_food = generate_food(dimension_grid, n_foods, food_quality, food_quantity, food_timing, food_duration, nests_coordinates)

	print position_food
	print quality_food
	print quantity_food
	print timing_food
	print duration_food
