#!/usr/bin/env python
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.patches import RegularPolygon
from function_model_v3 import *
from generate_food_distri import *
import json
import time
import argparse
import os
import datetime

def model(conf):

	n_nests = len(conf["nests_positions"])

	food_distribution = conf["food_initial_distribution"]
	
	if food_distribution == "fixed":
		n_foods = len(conf["foods_positions"])
		food_coordinates = np.array(conf["foods_positions"])
		food_quality = np.array(conf["food_quality"])
		food_quantity = np.array(conf["food_quantity"])
		food_timing = np.array(conf["food_timing"])
		food_duration = np.array(conf["food_duration"])
	elif food_distribution == "random":
		n_foods = conf["n_food_random"]
		food_coordinates, food_quality, food_quantity, food_timing, food_duration = generate_food(conf["grid_dimension"], n_foods, conf["food_quality"], conf["food_quantity"], conf["food_timing"], conf["food_duration"], conf["nests_positions"])
		food_coordinates = np.array(food_coordinates)
		food_quality = np.array(food_quality)
		food_quantity = np.array(food_quantity)
		food_timing = np.array(food_timing)
		food_duration = np.array(food_duration)


	nests_coordinates = np.array(conf["nests_positions"])
	n_ants_in_nest = conf["ants_in_nests"]

	field_connexion = conf["field_connexion"]

	dimension_grid = conf["grid_dimension"]
	time_simulation = conf["simulation_time"]

	record_at_foods = np.zeros(shape = (time_simulation, n_foods))
	record_at_nests = np.zeros(shape = (time_simulation, n_nests))
	
	evaporation_rate = conf["evaporation_rate"]

	p_sortie_k = conf["p_sortie_k"]
	p_sortie_a = conf["p_sortie_a"]
	p_sortie_b = conf["p_sortie_b"]

	p_out_food = conf["p_leave_food"]
	p_recruiter_leave_nest = conf["p_recruiter_leave_nest"]

	min_time_out = conf["minimum_time_out"]

	bias_1_good_cell_homing = np.array(conf["bias_1_good_cell_homing"])
	bias_2_good_cells_homing = np.array(conf["bias_2_good_cells_homing"])
	
	n_ants = np.sum(n_ants_in_nest)

	target = np.empty(shape = (n_ants, 3))

	nest_memory = np.empty(shape=(0,1))

	target_nests = np.empty(shape = (0, 3))
	ant_positions = np.empty(shape = (0,3))
	ant_directions = np.random.randint(6, size = (n_ants,))
	
	pheromone_deposit = np.zeros(shape =(n_ants,1))

	for j in range(n_nests):
		nest_memory = np.vstack((nest_memory, np.tile(j, (n_ants_in_nest[j], 1)) ))
		target_nests = np.vstack((target_nests, np.tile(nests_coordinates[j,:], (n_ants_in_nest[j], 1)) ))
		ant_positions = np.vstack((ant_positions, np.tile(nests_coordinates[j,:], (n_ants_in_nest[j], 1)) ))

	space_grid = np.ndarray(shape = ((dimension_grid*2-1)**2, 3))

	space_grid[:,0] = np.repeat(np.arange(-(dimension_grid-1), dimension_grid), dimension_grid*2-1)
	space_grid[:,1] = np.array(range(-(dimension_grid-1), dimension_grid) * (dimension_grid*2-1))
	space_grid[:,2] = -(space_grid[:,0] + space_grid[:,1])

	good_coord = (space_grid[:,2]<dimension_grid) & (space_grid[:,2]>-dimension_grid)

	foraging_area = space_grid[good_coord,:].astype('int')

	if field_connexion == "homogeneous":
		foraging_area_connexion = np.zeros(shape=(foraging_area.shape[0],6))
	elif filed_connexion == "heterogeneous":
		foraging_area_connexion = np.random.randint(5,25, size=(foraging_area.shape[0],6)) * 5

	ID_Cases = np.ndarray(shape = (dimension_grid*2+1,dimension_grid*2+1,dimension_grid*2+1)).astype('int')

	for i in range(0, foraging_area.shape[0]):
		ID_Cases[foraging_area[i,0], foraging_area[i,1], foraging_area[i,2]] = i

	ID_cases = convert_to_ID(foraging_area, dimension_grid-1, dimension_grid*2-1)

	pheromone = np.zeros(shape = (foraging_area.shape[0], 1))
	homing_status = np.zeros(shape = (n_ants, 1))
	food_memory = np.empty(shape = (n_ants, 1))
	food_memory[:] = np.nan

	time_out = np.zeros(shape = (n_ants, )) -1

	at_food_or_nest = np.ones(shape = (n_ants, ))

	food_carrier = np.zeros(shape = (n_ants, ))
	first_left = np.zeros(shape = (n_ants, ))
	is_recruiting = np.zeros(shape = (n_ants, ))

	total_food = np.zeros(shape = (n_foods,))

	path = np.zeros(shape = (foraging_area.shape[0], 1))
	limits = np.sum(np.abs(foraging_area) == dimension_grid-1, axis = 1) == 0

	biais_zeros = np.ones(shape=(1, 6))
	biais_move = np.array([100,20,2,1,2,20])

	biais_direction = np.empty(shape = (n_ants, 6))

	ponderation_phero = np.tile(np.array([1., 1., 0., 0., 0., 1.]), (n_ants, 1))
	
	#biais_direction = np.tile(np.array([1,1,1,1,1,1]), (n_ants,1))

	potential_moves = np.array([[0, 1, -1],[1, 0, -1],[1, -1, 0],[0, -1, 1],[-1, 0, 1],[-1, 1, 0]])

	at_food = np.zeros(shape = (n_ants,))
	at_nest = np.ones(shape = (n_ants,))

	start_time = time.time()
	
	record_positions_ID = np.ndarray(shape =(time_simulation, n_ants))
	
	for t in range(time_simulation):
		
		#p_exit_nest = compute_p_out_pheromone(pheromone, nests_coordinates, dimension_grid-1, dimension_grid*2-1, ID_cases, p_sortie_k, p_sortie_a, p_sortie_b)
		
		p_exit_nest = compute_p_out_recruiters(n_nests, is_recruiting, nest_memory, p_sortie_k, p_sortie_a, p_sortie_b)
		
		biais_direction[at_food_or_nest==1,:] = biais_zeros
		biais_direction[at_food_or_nest==0,:] = biais_move
		
		movement_or_not, is_recruiting = check_movement(at_food, at_nest, is_recruiting, p_out_food, p_recruiter_leave_nest, p_exit_nest, food_memory, nest_memory)
		
		homing_status[(at_nest==1) & (movement_or_not==1)] = 0
		
		food_taken = np.unique(food_memory[(at_food==1) & (movement_or_not==1) & (first_left==0)], return_counts = True)
		food_quantity[food_taken[0].astype('int')] -= food_taken[1]
		
		first_left[(food_carrier == 1) & (first_left==0) & (at_food==1) & (movement_or_not==1)] = 1
		
		moves = select_move(ant_positions, ant_directions, biais_direction, bias_1_good_cell_homing, bias_2_good_cells_homing, foraging_area_connexion, pheromone, homing_status, target, potential_moves, ponderation_phero, limits, ID_cases, ID_Cases, dimension_grid-1, dimension_grid*2-1)

		time_out[(at_nest==1) & movement_or_not==1] = min_time_out
		
		ant_positions[movement_or_not==1,:] += potential_moves[moves[movement_or_not==1],:]
		
		ant_directions = moves
		
		homing_status, pheromone_deposit, food_memory, nest_memory, target, at_food_or_nest, at_food, at_nest, food_carrier, first_left, is_recruiting, time_out, total_food, record_at_foods, record_at_nests = check_homing(homing_status, pheromone_deposit, food_memory, nest_memory, target, ant_positions, food_coordinates, n_foods, nests_coordinates, n_nests, food_quality, food_quantity, food_timing, food_duration, food_carrier, first_left, is_recruiting, time_out, total_food, t, record_at_foods, record_at_nests)
		
		positions_ant_ID = convert_to_ID(ant_positions, dimension_grid-1, dimension_grid*2-1)
		
		record_positions_ID[t,:] = positions_ant_ID
		
		pheromone = update_path(pheromone, positions_ant_ID, ID_cases, homing_status * np.reshape((1- is_recruiting),(n_ants,1)), pheromone_deposit)
		
		pheromone = pheromone * (1-evaporation_rate)
		
		time_out[at_nest==0] -= 1
		
		path = update_path(path, positions_ant_ID, ID_cases, np.ones(shape = (homing_status.shape)), np.ones(shape = (pheromone_deposit.shape)))


			#print movement_or_not
#	print time.time() - start_time
#	print total_food
#	print pheromone[pheromone> np.percentile(pheromone, 95)]

	return foraging_area, path, pheromone, total_food, record_at_foods, record_at_nests, record_positions_ID, nests_coordinates, food_coordinates, food_quality, n_ants_in_nest


if __name__ == '__main__':
	
	ap = argparse.ArgumentParser()
	ap.add_argument("-c", "--conf", help="config file")
	args = vars(ap.parse_args())

	conf = json.load(open(args["conf"]))

	output_dir = os.path.join(os.getcwd(), datetime.datetime.now().strftime('%Y-%m-%d_%H-%M-%S'))
	os.makedirs(output_dir)
	
	foraging_area, path, pheromone, food, ants_at_foods, ants_at_nests, record_positions_ID = model(conf)
