#!/usr/bin/env python
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.patches import RegularPolygon
from function_model_v3 import *
import json
import time
import argparse
import os
import datetime
from main_v3 import *
from shutil import copyfile

ap = argparse.ArgumentParser()
ap.add_argument("-c", "--conf", help="config file")
args = vars(ap.parse_args())

conf = json.load(open(args["conf"]))

output_dir = os.path.join(os.getcwd(), datetime.datetime.now().strftime('%Y-%m-%d_%H-%M-%S'))
os.makedirs(output_dir)

copyfile(args["conf"], output_dir + '/conf.json')

total_food = []

n_sim = conf["n_simulations"]

for i in range(0, n_sim):
	
	foraging_area, path, pheromone, food, ants_at_foods, ants_at_nests, record_positions_ID, nests_coordinates, food_coordinates, food_quality, n_ants_in_nest = model(conf)
	total_food.append(food)
	
	plot_map(foraging_area, path, pheromone, nests_coordinates, food_coordinates, food_quality, n_ants_in_nest, i, output_dir)

	np.savetxt(output_dir + '/simulation_' + str(i) + '.txt', record_positions_ID.astype('int'), fmt = '%d')

	np.savetxt(output_dir + '/simulation_' + str(i) + '_ants_at_food.txt', ants_at_foods.astype('int'), fmt = '%d')
	
	np.savetxt(output_dir + '/simulation_' + str(i) + '_ants_at_nests.txt', ants_at_nests.astype('int'), fmt = '%d')
	
	plot_dynamics(ants_at_foods, ants_at_nests, i, output_dir)

	print "[INFO] Simulation " + str(i+1) + "/" + str(n_sim) + " done"

np.savetxt(output_dir + '/food_collected.txt', total_food, fmt = '%d')
